package org.fuseri.modulelanguageschool.lecture;

import org.fuseri.model.dto.course.LanguageTypeDto;
import org.fuseri.model.dto.course.ProficiencyLevelDto;
import org.fuseri.model.dto.lecture.LectureCreateDto;
import org.fuseri.model.dto.lecture.LectureDto;
import org.fuseri.model.dto.user.UserDto;
import org.fuseri.modulelanguageschool.course.CourseService;
import org.fuseri.modulelanguageschool.course.Language;
import org.fuseri.modulelanguageschool.course.ProficiencyLevel;
import org.fuseri.modulelanguageschool.user.UserMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
@Transactional
public class LectureFacade {

    private final LectureService lectureService;
    private final LectureMapper lectureMapper;
    private final UserMapper userMapper;
    private final CourseService courseService;

    @Autowired
    public LectureFacade(LectureService lectureService, LectureMapper lectureMapper, UserMapper userMapper, CourseService courseService) {
        this.lectureService = lectureService;
        this.lectureMapper = lectureMapper;
        this.userMapper = userMapper;
        this.courseService = courseService;
    }

    @Transactional
    public LectureDto create(LectureCreateDto dto) {
        return lectureMapper.mapToDto(lectureService.save(lectureMapper.mapToLecture(dto, courseService)));
    }

    @Cacheable(cacheNames = "courses", key = "#id")
    @Transactional(readOnly = true)
    public LectureDto findById(Long id) {
        return lectureMapper.mapToDto(lectureService.findById(id));
    }

    @Transactional(readOnly = true)
    public List<LectureDto> findAll(Long id) {
        return lectureMapper.mapToList(lectureService.findAllByCourse(id));
    }

    @Transactional
    public LectureDto update(Long id, LectureCreateDto dto) {
        return lectureMapper.mapToDto(lectureService.update(id, lectureMapper.mapToLecture(dto, courseService)));
    }

    @Transactional
    public void delete(Long id) {
        lectureService.delete(id);
    }

    public List<LectureDto> findAll(LanguageTypeDto lang) {
        return lectureMapper.mapToList(lectureService.findAll(Language.valueOf(lang.name())));
    }

    public List<LectureDto> findAll(LanguageTypeDto lang, ProficiencyLevelDto prof) {
        return lectureMapper.mapToList(lectureService.findAll(Language.valueOf(lang.name()), ProficiencyLevel.valueOf(prof.name())));
    }

    public LectureDto enrol(Long id, UserDto student) {
        return lectureMapper.mapToDto(lectureService.enrol(id, userMapper.fromDto(student)));
    }

    public LectureDto expel(Long id, UserDto student) {
        return lectureMapper.mapToDto(lectureService.expel(id, userMapper.fromDto(student)));
    }

    public LectureDto setLecturer(Long id, UserDto lecturerDto) {
        return lectureMapper.mapToDto(lectureService.setLecturer(id, userMapper.fromDto(lecturerDto)));
    }
}
